"""Configuration."""

import json
import os


class Config(object):
    """
    Get all configuration from the config.json file.
    If config.json dose not exist, use config.example.json file.

        Note:   Environment variables have a higher priority,
                if you set a environment variable in your system,
                that variable in the config.json file will be invalid.
    """

    def __init__(self):
        # Open and read the config file
        # project_path = os.path.dirname(os.path.dirname(__file__))
        project_path = os.path.dirname(__file__)
        config_file = os.path.join(project_path, 'config', 'config.json')
        if not os.path.exists(config_file):
            config_file = os.path.join(project_path, 'config', 'config.example.json')

        with open(config_file, 'r', encoding='utf-8') as f:
            config_json = json.load(f)

        # Language
        self.LANGUAGE = os.environ['LANGUAGE'] if os.environ.get('LANGUAGE') else config_json['language']

        # Cookie configs
        # Cookie from https://bbs.mihoyo.com/ys/
        self.COOKIE_MIHOYOBBS = os.environ['COOKIE_MIHOYOBBS'] if os.environ.get('COOKIE_MIHOYOBBS') else config_json['cookies']['cookie_mihoyobbs']

        # Cookie from https://www.hoyolab.com/genshin/
        self.COOKIE_HOYOLAB = os.environ['COOKIE_HOYOLAB'] if os.environ.get('COOKIE_HOYOLAB') else config_json['cookies']['cookie_hoyolab']

        # Cookie from https://m.weibo.cn
        self.COOKIE_WEIBO = os.environ['COOKIE_WEIBO'] if os.environ.get('COOKIE_WEIBO') else config_json['cookies']['cookie_weibo']
        self.WEIBO_INTL_AID = os.environ['WEIBO_INTL_AID'] if os.environ.get('WEIBO_INTL_AID') else config_json['cookies']['weibo_intl_aid']
        self.WEIBO_INTL_S = os.environ['WEIBO_INTL_S'] if os.environ.get('WEIBO_INTL_S') else config_json['cookies']['weibo_intl_s']

        # Cookie from https://ka.sina.com.cn
        self.COOKIE_KA = os.environ['COOKIE_KA'] if os.environ.get('COOKIE_KA') else config_json['cookies']['cookie_ka']

        # Notifier configs
        # iOS Bark App
        self.BARK_KEY = config_json['notifiers']['bark_key']
        if os.environ.get('BARK_KEY'):
            # Customed server
            if os.environ['BARK_KEY'].find(
                    'https') != -1 or os.environ['BARK_KEY'].find('http') != -1:
                self.BARK_KEY = os.environ['BARK_KEY']
            else:
                self.BARK_KEY = f"https://api.day.app/{os.environ['BARK_KEY']}"
        # Official server
        elif self.BARK_KEY and self.BARK_KEY.find('https') == -1 and self.BARK_KEY.find('http') == -1:
            self.BARK_KEY = f'https://api.day.app/{self.BARK_KEY}'

        self.BARK_SOUND = os.environ['BARK_SOUND'] if os.environ.get('BARK_SOUND') else config_json['notifiers'][
            'bark_sound']

        # Cool Push
        self.COOL_PUSH_SKEY = os.environ['COOL_PUSH_SKEY'] if os.environ.get('COOL_PUSH_SKEY') else config_json['notifiers']['cool_push_skey']
        self.COOL_PUSH_MODE = os.environ['COOL_PUSH_MODE'] if os.environ.get('COOL_PUSH_MODE') else config_json['notifiers']['cool_push_mode']

        # Custom Notifier Config
        self.CUSTOM_NOTIFIER = os.environ['CUSTOM_NOTIFIER'] if os.environ.get('CUSTOM_NOTIFIER') else config_json['notifiers']['custom_notifier']

        # DingTalk Bot
        self.DD_BOT_TOKEN = os.environ['DD_BOT_TOKEN'] if os.environ.get('DD_BOT_TOKEN') else config_json['notifiers']['dingtalk_bot_token']
        self.DD_BOT_SECRET = os.environ['DD_BOT_SECRET'] if os.environ.get('DD_BOT_SECRET') else config_json['notifiers']['dingtalk_bot_secret']

        # Discord webhook
        self.DISCORD_WEBHOOK = os.environ['DISCORD_WEBHOOK'] if os.environ.get('DISCORD_WEBHOOK') else config_json['notifiers']['discord_webhook']

        # iGot
        self.IGOT_KEY = os.environ['IGOT_KEY'] if os.environ.get('IGOT_KEY') else config_json['notifiers']['igot_key']

        # pushplus
        self.PUSH_PLUS_TOKEN = os.environ['PUSH_PLUS_TOKEN'] if os.environ.get('PUSH_PLUS_TOKEN') else config_json['notifiers']['push_plus_token']
        self.PUSH_PLUS_USER = os.environ['PUSH_PLUS_USER'] if os.environ.get('PUSH_PLUS_USER') else config_json['notifiers']['push_plus_user']

        # Server Chan
        self.SCKEY = os.environ['SCKEY'] if os.environ.get('SCKEY') else config_json['notifiers']['server_chan_key']
        self.SCTKEY = os.environ['SCTKEY'] if os.environ.get('SCTKEY') else config_json['notifiers']['server_chan_turbo_key']

        # Telegram Bot
        self.TG_BOT_API = os.environ['TG_BOT_API'] if os.environ.get('TG_BOT_API') else config_json['notifiers']['telegram_bot_api']
        self.TG_BOT_TOKEN = os.environ['TG_BOT_TOKEN'] if os.environ.get('TG_BOT_TOKEN') else config_json['notifiers']['telegram_bot_token']
        self.TG_USER_ID = os.environ['TG_USER_ID'] if os.environ.get('TG_USER_ID') else config_json['notifiers']['telegram_user_id']

        # WeChat Work App
        self.WW_ID = os.environ['WW_ID'] if os.environ.get('WW_ID') else config_json['notifiers']['wechat_work_id']
        self.WW_APP_SECRET = os.environ['WW_APP_SECRET'] if os.environ.get('WW_APP_SECRET') else config_json['notifiers']['wechat_work_app_secret']
        self.WW_APP_USERID = os.environ['WW_APP_USERID'] if os.environ.get('WW_APP_USERID') else config_json['notifiers']['wechat_work_app_userid']
        self.WW_APP_AGENTID = os.environ['WW_APP_AGENTID'] if os.environ.get('WW_APP_AGENTID') else config_json['notifiers']['wechat_work_app_agentid']

        # WeChat Work Bot
        self.WW_BOT_KEY = os.environ['WW_BOT_KEY'] if os.environ.get('WW_BOT_KEY') else config_json['notifiers']['wechat_work_bot_key']


config = Config()



class GenshinHelperException(Exception):
    """Base genshinhelper exception."""


class CookiesExpired(GenshinHelperException):
    """Cookies has expired."""


class NotificationError(GenshinHelperException):
    """
    A notification error. Raised after an issue with the sent notification.
    """


class NoSuchNotifierError(GenshinHelperException):
    """
    An unknown notifier was requests, one that was not registered.
    """



from .basenotifier import BaseNotifier as Base
from ..config import config


class CoolPush(Base):
    def __init__(self):
        self.name = 'Cool Push'
        self.token = config.COOL_PUSH_SKEY
        self.retcode_key = 'code'
        self.retcode_value = 200

    def send(self, text=Base.app, status=Base.status, desp=Base.desp):
        url = f'https://push.xuthus.cc/{config.COOL_PUSH_MODE}/{config.COOL_PUSH_SKEY}'
        data = f'{text} {status}\n\n{desp}'.encode('utf-8')
        return self.push('post', url, data=data)



import re

from bs4 import BeautifulSoup

from .config import config
from .exceptions import CookiesExpired
from .utils import log, request


class SuperTopicCheckin(object):
    USER_AGENT = 'WeiboOverseas/4.1.4 (iPhone; iOS 14.0.1; Scale/2.00)'
    WEIBO_API = 'https://api.weibo.cn'
    FOLLOW_DATA_URL = WEIBO_API + '/2/cardlist'
    SIGN_URL = WEIBO_API + '/2/page/button'

    def __init__(self, cookie: str = None):
        self._cookie = cookie

    def get_header(self):
        header = {
            'User-Agent': self.USER_AGENT,
            'Cookie': self._cookie
        }
        return header

    def get_follow_data(self):
        try:
            url = self.FOLLOW_DATA_URL
            data = {
                'aid': config.WEIBO_INTL_AID,
                'c': 'weicoabroad',
                'containerid': '100803_-_followsuper',
                's': config.WEIBO_INTL_S,
                'ua': 'iPhone12%2C1_iOS14.0.1_Weibo_intl_4140_cell'
            }
            response = request(
                'get',
                url,
                params=data,
                headers=self.get_header(),
                verify=False).json()
        except Exception as e:
            raise Exception(f'Failed to get follow list:\n{e}')
        else:
            card_group = response.get('cards', [{}])[0].get('card_group', [])
            follow_list = [
                card for card in card_group if card.get('card_type') == '8'
            ]
            if not follow_list:
                return []

            follow_data = []
            for item in follow_list:
                action = item.get('buttons', [{}])[0].get('params', {}).get(
                    'action', '')
                request_url = ''.join(
                    re.findall('(?<=request_url=)(.*)%26container', action))
                sign_data = {k: v for (k, v) in data.items()}
                del sign_data['containerid']
                sign_data['request_url'] = request_url

                follow = {
                    'name': item.get('title_sub'),
                    'lv': int(re.findall('\d+', item.get('desc1', '0'))[0]),
                    'is_sign': item.get('buttons', [{}])[0].get('name'),
                    'sign_data': sign_data
                }
                follow_data.append(follow)

            follow_data.sort(key=lambda k: (k.get('lv', 0)), reverse=True)

            return follow_data

    def run(self):
        follow_data = self.get_follow_data()
        if not follow_data:
            raise Exception('Empty follow data')

        message = []
        for follow in follow_data:
            name = follow.get('name')
            lv = follow.get('lv')
            is_sign = follow.get('is_sign')
            data = follow.get('sign_data', {})
            name_style = f'⚜️ [Lv.{lv}]{name}'

            # Already checked in
            if is_sign == '已签':
                message.append(f'{name_style} ☑️')
            elif is_sign == '签到':
                try:
                    url = self.SIGN_URL
                    response = request(
                        'get', url, params=data, headers=self.get_header(), verify=False).json()
                except Exception as e:
                    raise Exception(f'Failed to run sign:\n{e}')
                else:
                    if response.get('result') in (1, '1'):
                        message.append(f'{name_style} ✅')
                    # Already checked in
                    elif '已签到' in response.get('msg', ''):
                        message.append(f'{name_style} ☑️')
                    # Verification required
                    # elif response.get('result') == 0 and response.get('scheme'):
                    #     log.error(f'Failed to run sign:\nVerification required.')
                    else:
                        raise Exception(f'{name_style} :Failed to run sign:\n{response}')

            else:
                log.info('Unknown check-in status')

        return '\n    '.join(message)


class RedemptionCode(object):
    USER_AGENT = 'WeiboOverseas/4.1.4 (iPhone; iOS 14.0.1; Scale/2.00)'
    CONTAINER_ID = '100808fc439dedbb06ca5fd858848e521b8716'
    GENSHIN_CHAOHUA_URL = 'https://m.weibo.cn/api/container/getIndex?containerid={}_-_feed'.format(CONTAINER_ID)
    DRAW_URL = 'https://ka.sina.com.cn/innerapi/draw'
    MYBOX_URL = 'https://ka.sina.com.cn/html5/mybox'

    def __init__(self, cookie: str = None):
        self._cookie = cookie

    def get_header(self):
        header = {
            'User-Agent': self.USER_AGENT,
            'Referer': 'https://m.weibo.cn',
            'Cookie': self._cookie
        }
        return header

    @property
    def event_gift_ids(self):
        try:
            url = self.GENSHIN_CHAOHUA_URL
            response = request('get', url, headers=self.get_header()).json()
        except Exception as e:
            raise Exception(f'Failed to get event information:\n{e}')
        else:
            cards = response.get('data', {}).get('cards', [])
            event_card = [
                card for card in cards if card.get('itemid') == 'pagemanual_2'
            ]
            if not event_card:
                return []

            group = event_card[0].get('card_group', [{}])[0].get('group', [])

            ids = [i for item in group if '签到' in item.get('title_sub') for i in re.findall('gift/(\d*)', item['scheme'])]
            return ids

    @property
    def mybox_codes(self):
        header = self.get_header()
        header['Referer'] = 'https://ka.sina.com.cn/html5/'
        try:
            url = self.MYBOX_URL
            response = request(
                'get', url, headers=header, allow_redirects=False)
        except Exception as e:
            raise Exception(f'Failed to get mybox codes:\n{e}')
        else:
            mybox_codes = []
            if response.status_code == 200:
                response.encoding = 'utf-8'
                soup = BeautifulSoup(response.text, 'html.parser')
                # print(soup.prettify())
                boxs = soup.find_all(class_='giftbag')

                for box in boxs:
                    item = {
                        'id': box.find(class_='deleBtn').get('data-itemid'),
                        'title': box.find(class_='title itemTitle').text,
                        'code': box.find('span').parent.contents[1]
                    }
                    mybox_codes.append(item)

            elif response.status_code == 302 or 500:
                raise CookiesExpired('😳 ka.sina: 登录可能失效, 尝试重新登录')
            else:
                raise Exception('😳 ka.sina: 个人中心兑换码获取失败')
            return mybox_codes

    @property
    def mybox_gift_ids(self):
        ids = [item.get('id') for item in self.mybox_codes]
        return ids

    @property
    def unclaimed_gift_ids(self):
        event_gift_ids = self.event_gift_ids
        if not event_gift_ids:
            return []

        mybox_gift_ids = self.mybox_gift_ids
        ids = [i for i in event_gift_ids if i not in mybox_gift_ids]
        return ids

    def get_code(self, id):
        header = self.get_header()
        header['Referer'] = f'https://ka.sina.com.cn/html5/gift/{id}'
        data = {'gid': 10725, 'itemId': id, 'channel': 'wblink'}

        try:
            url = self.DRAW_URL
            response = request(
                'get', url, params=data, headers=header).json()
        except Exception as e:
            raise Exception(f'Failed to get redemption code with ID {id}:\n{e}')
        else:
            if response.get('k'):
                log.info(f'{id} 的兑换码领取成功')
                return response['data']['kahao']
            elif response.get('code') == '2002' and '头像' in response.get(
                    'msg', ''):
                raise Exception(f'🥳 {id}: 每天只能领取一张或该兑换码已经领取过了哦')
            elif response.get('code') == '2002':
                raise Exception(f'😳 {id}: {response["msg"]}')
            elif 'login' in response.get('msg', ''):
                raise Exception('登录失效, 请重新登录')
            else:
                raise Exception(f'😳 {id}: {response}')

    def run(self):
        ids = self.unclaimed_gift_ids
        codes = []
        if not ids:
            msg = '没有未领取的兑换码'
            log.info(msg)
            codes.append(msg)
        else:
            log.info(f'检测到有 {len(ids)} 个未领取的兑换码')

            for id in ids:
                code = self.get_code(id)
                if code:
                    codes.append(code)
        return '\n    '.join(codes)



"""Utilities."""

import logging

import requests
from requests.exceptions import HTTPError

logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s %(levelname)s %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S')

log = logger = logging


def request(method: str,
            url: str,
            max_retry: int = 2,
            params=None,
            data=None,
            json=None,
            headers=None,
            **kwargs):
    for i in range(max_retry + 1):
        try:
            response = requests.Session().request(
                method,
                url,
                params=params,
                data=data,
                json=json,
                headers=headers,
                **kwargs)
        except HTTPError as e:
            log.error(f'HTTP error:\n{e}')
            log.error(f'The No.{i + 1} request failed, retrying...')
        except KeyError as e:
            log.error(f'Wrong response:\n{e}')
            log.error(f'The No.{i + 1} request failed, retrying...')
        except Exception as e:
            log.error(f'Unknown error:\n{e}')
            log.error(f'The No.{i + 1} request failed, retrying...')
        else:
            return response

    raise Exception(f'All {max_retry + 1} HTTP requests failed, die.')


def get_cookies(cookies: str = None):
    if '#' in cookies:
        return cookies.split('#')
    else:
        return cookies.splitlines()


MESSAGE_TEMPLATE = '''
    {today:#^18}
    🔅[{region_name}]{uid}
    今日奖励: {award_name} × {award_cnt}
    本月累签: {total_sign_day} 天
    签到结果: {status}
    {end:#^18}'''



from urllib import parse

from .basenotifier import BaseNotifier as Base
from ..config import config


class Bark(Base):
    def __init__(self):
        self.name = 'Bark App'
        self.token = config.BARK_KEY
        self.retcode_key = 'code'
        self.retcode_value = 200

    def send(self, text=Base.app, status=Base.status, desp=Base.desp):
        url = f'{config.BARK_KEY}/{text} {status}/{parse.quote(str(desp))}'
        data = {'sound': config.BARK_SOUND}
        return self.push('get', url, params=data)



from .basenotifier import BaseNotifier as Base
from ..config import config


class WechatWorkBot(Base):
    def __init__(self):
        self.name = 'Wechat Work Bot'
        self.token = config.WW_BOT_KEY
        self.retcode_key = 'errcode'
        self.retcode_value = 0

    def send(self, text=Base.app, status=Base.status, desp=Base.desp):
        url = f'https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key={config.WW_BOT_KEY}'
        data = {
            'msgtype': 'text',
            'text': {
                'content': f'{text} {status}\n\n{desp}'
            }
        }
        return self.push('post', url, json=data)
